/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/content/NodeRevisionNumber.java,v 1.3 2000/12/01 07:17:30 remm Exp $
 * $Revision: 1.3 $
 * $Date: 2000/12/01 07:17:30 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.content;

import java.io.Serializable;
import java.util.StringTokenizer;
import org.apache.slide.common.ObjectValidationFailedException;
import org.apache.slide.util.Messages;

/**
 * Node Revision Number class.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.3 $
 */
public final class NodeRevisionNumber implements Serializable, Cloneable {
    
    
    // -------------------------------------------------------------- Constants
    
    
    public static final int MAJOR = 0;
    public static final int MINOR = 1;
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Create a new revision number.
     */
    public NodeRevisionNumber() {
        number = new int[2];
        number[MAJOR] = 1;
        number[MINOR] = 0;
    }
    
    
    /**
     * Create a new revision number.
     * 
     * @param major 
     * @param minor 
     */
    public NodeRevisionNumber(int major, int minor) {
        number = new int[2];
        number[MAJOR] = major;
        number[MINOR] = minor;
    }
    
    
    /**
     * Create a new revision number.
     * 
     * @param revisionNumberStr String representation of the revision number
     */
    public NodeRevisionNumber(String revisionNumberStr) {
        parseString(revisionNumberStr);
    }
    
    
    /**
     * Create a new revision number based on a previous revision number.
     * 
     * @param revisionNumber Previous revision number
     */
    public NodeRevisionNumber(NodeRevisionNumber revisionNumber) {
        this(revisionNumber, false);
    }
    
    
    /**
     * Create a new revision number based on a previous revision number.
     * 
     * @param revisionNumber Previous revision number
     * @param createBranch True if a new branch is to be created
     */
    public NodeRevisionNumber(NodeRevisionNumber revisionNumber, 
                              boolean createBranch) {
        if (revisionNumber != null) {
            if (createBranch) {
                parseString(revisionNumber.toString() + ".1");
            } else {
                parseString(revisionNumber.toString());
                next();
            }
        } else {
            number = new int[2];
            number[MAJOR] = 1;
            number[MINOR] = 0;
        }
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Number storage.
     */
    private int[] number;
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Get the first number in the revision number.
     * 
     * @return int 
     */
    public int getMajor() {
        return number[MAJOR];
    }
    
    
    /**
     * Get the second number in the revision number.
     * 
     * @return int 
     */
    public int getMinor() {
        return number[MINOR];
    }
    
    
    /**
     * Get a number by specifying its order.
     * 
     * @param pos Position of the number
     * @return int 
     */
    public int getNumber(int pos) {
        return number[pos];
    }
    
    
    /**
     * Return number of digits if the revision number.
     * 
     * @return int
     */
    public int getNbDigits() {
        return number.length;
    }
    
    
    // -------------------------------------------------------- Private Methods
    
    
    /**
     * Parse a String.
     * 
     * @param revisionNumberStr String representation of the revision number
     */
    private void parseString(String revisionNumberStr) {
        // We tokenize the string using "." as a delimiter.
        StringTokenizer tokenizer = 
            new StringTokenizer(revisionNumberStr, ".");
        int nbDigits = tokenizer.countTokens();
        if (nbDigits > 0) {
            number = new int[nbDigits];
            for (int i=0; i<nbDigits; i++) {
                number[i] = (new Integer(tokenizer.nextToken())).intValue();
            }
        } else {
            number = new int[2];
            number[MAJOR] = 1;
            number[MINOR] = 0;
        }
    }
    
    
    /**
     * Next revision.
     */
    private void next() {
        if (number.length > 0) {
            number[number.length - 1] += 1;
        }
    }
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Get a String representation of the revision number.
     * 
     * @return String 
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append(number[MAJOR]);
        for (int i=1; i<number.length; i++) {
            buf.append(".").append(number[i]);
        }
        return buf.toString();
    }
    
    
    /**
     * HashCode.
     * 
     * @return int 
     */
    public int hashCode() {
        return toString().hashCode();
    }
    
    
    /**
     * Equals.
     * 
     * @param obj Object to test
     * @return boolean True if the two object are equal :
     * <li>obj is of type NodeRevisionNumber and is not null</li>
     * <li>The string representations are equal are equal</li>
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if ((obj != null) && (obj instanceof NodeRevisionNumber)) {
            result = (this.toString().equals(obj.toString()));
        }
        return result;
    }
    
    
    /**
     * Clone.
     * 
     * @return Object clone
     */
    NodeRevisionNumber cloneObject() {
        NodeRevisionNumber result = null;
        try {
            result = (NodeRevisionNumber) super.clone();
        } catch(CloneNotSupportedException e) {
        }
        return result;
    }
    
    
    /**
     * Validate.
     */
    public void validate() {
        
        if (number == null)
            throw new ObjectValidationFailedException
                (Messages.message
                 (NodeRevisionNumber.class.getName() + ".nullNumber"));
        
        if (number.length < 2)
            throw new ObjectValidationFailedException
                (Messages.message
                 (NodeRevisionNumber.class.getName() + ".invalidNumber"));
        
    }
    
    
}
