/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/structure/Structure.java,v 1.8 2001/08/20 15:47:44 cmlenz Exp $
 * $Revision: 1.8 $
 * $Date: 2001/08/20 15:47:44 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.structure;

import java.io.*;
import java.util.Enumeration;
import java.util.Stack;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Date;
import org.apache.slide.common.*;
import org.apache.slide.lock.*;
import org.apache.slide.security.*;
import org.apache.slide.authenticate.CredentialsToken;

/**
 * Structure helper.
 * Provides methods to manage and navigate the hierarchy of nodes in a 
 * namespace.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.8 $
 */
public interface Structure {
    
    
    // ------------------------------------------------------ Interface Methods
    
    
    /**
     * Returns the children of a node.
     * 
     * @param token the slide token
     * @param object the node
     * 
     * @return Enumeration of ObjectNode objects
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectNotFoundException one of the children specified 
     *            by the object was not found
     * @exception LinkedObjectNotFoundException cannot happen
     *
     * @see org.apache.slide.structure.ObjectNode#enumerateChildren()
     */
    Enumeration getChildren(SlideToken token, ObjectNode object) 
        throws ServiceAccessException, ObjectNotFoundException, 
        LinkedObjectNotFoundException;
    
    
    /**
     * Returns the parent of a node.
     * 
     * @param token the slide token
     * @param object the node
     * 
     * @return the node's parent, or <code>null</code> if the object specified 
     *         is the root node of the namespace
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectNotFoundException the parent object specified 
     *            by the object was not found
     * @exception LinkedObjectNotFoundException cannot happen (a link 
     *            cannot have children)
     * @exception AccessDeniedException credentials token does not 
     *            have permission to perform the action
     *
     * @see org.apache.slide.structure.ObjectNode#enumerateChildren()
     */
    ObjectNode getParent(SlideToken token, ObjectNode object) 
        throws ServiceAccessException, ObjectNotFoundException, 
        LinkedObjectNotFoundException, AccessDeniedException;
    
    
    /**
     * Retrieves a node by URI, following any links.
     * 
     * @param token the slide token
     * @param strUri the URI of the object to retrieve
     * 
     * @return the node specified by the URI or the node linked to by the node
     *         at the URI.
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectNotFoundException if no node exists at the URI
     * @exception LinkedObjectNotFoundException A link object encountered 
     *            during URI resolution doesn't have a valid target
     * @exception AccessDeniedException credentials token does not have 
     *            permission to perform the action
     */
    ObjectNode retrieve(SlideToken token, String strUri) 
        throws ServiceAccessException, ObjectNotFoundException, 
        LinkedObjectNotFoundException, AccessDeniedException;
    
    
    /**
     * Retrieves a node by URI.
     * 
     * @param token the slide token
     * @param strUri the URI of the object to retrieve
     * @param translateLastUriElement if set to true and the URI is 
     *        associated with a link, this method will return the target of 
     *        the link, otherwise the link itself is returned
     * 
     * @return the node specified by the URI, or the node linked to by the
     *         node at the URI, if <code>translateLastUriElement</code> is true
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectNotFoundException if no node exists at the URI
     * @exception LinkedObjectNotFoundException A link object encountered 
     *            during URI resolution doesn't have a valid target
     * @exception AccessDeniedException credentials token does not have 
     *            permission to perform the action
     */
    ObjectNode retrieve(SlideToken token, String strUri, 
                        boolean translateLastUriElement) 
        throws ServiceAccessException, ObjectNotFoundException, 
        LinkedObjectNotFoundException, AccessDeniedException;
    
    
    /**
     * Creates a new node in the namespace.
     * 
     * @param token the slide token
     * @param object the object that should be created
     * @param strUri location in the namespace where we the object should be 
     *        created
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectAlreadyExistException an object already exists 
     *            at the specified URI
     * @exception ObjectNotFoundException Update of the parent object 
     *            failed because the parent object does no longer exist. 
     *            Should not happen, and indicates a critical error if it does
     * @exception LinkedObjectNotFoundException retrieval of a link target 
     *            failed during URI resolution
     * @exception AccessDeniedException credentials token does not have 
     *            permission to perform the action
     */
    void create(SlideToken token, ObjectNode object, String strUri)
        throws ServiceAccessException, ObjectAlreadyExistsException, 
        ObjectNotFoundException, LinkedObjectNotFoundException, 
        AccessDeniedException;
    
    
    /**
     * Creates a link to another node in the namespace.
     * 
     * @param token the slide token
     * @param link the link object that should be created
     * @param linkUri location in the namespace where the link object should 
     *        be created
     * @param linkedObject target object of the link
     * 
     * @exception ServiceAccessException Low level service access exception
     * @exception ObjectAlreadyExistException An object already exist 
     *            at the specified URI
     * @exception ObjectNotFoundException Update of the parent object 
     *            failed because the parent object does no longer exist. 
     *            Should not happen, and indicate a critical error if it does
     * @exception LinkedObjectNotFoundException Retrieval of a link target 
     *            failed during Uri resolution
     * @exception AccessDeniedException credentials token does not have 
     *            permission to perform the action
     */
    void createLink(SlideToken token, LinkNode link, String linkUri, 
                    ObjectNode linkedObject)
        throws ServiceAccessException, ObjectAlreadyExistsException, 
        ObjectNotFoundException, LinkedObjectNotFoundException, 
        AccessDeniedException;
    
    
    /**
     * Stores/updates an object.
     * 
     * @param token the slide token
     * @param object the object to update
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectNotFoundException the update failed because one 
     *            object was not found during URI resolution
     * @exception LinkedObjectNotFoundException retrieval of a link target 
     *            failed during URI resolution
     * @exception AccessDeniedException credentials token does not have 
     *            permission to perform the action
     */
    void store(SlideToken token, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException, 
        AccessDeniedException, LinkedObjectNotFoundException;
    
    
    /**
     * Removes a node from the namespace.
     * 
     * @param token the slide token
     * @param object the node to remove
     * 
     * @exception ServiceAccessException low level service access exception
     * @exception ObjectNotFoundException the update failed because an 
     *            object was not found during URI resolution
     * @exception ObjectHasChildrenException removal failed because object 
     *            has children
     * @exception LinkedObjectNotFoundException retrieval of a link target 
     *            failed during URI resolution
     * @exception AccessDeniedException credentials token does not have 
     *            permission to perform the action
     */
    void remove(SlideToken token, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException, 
        ObjectHasChildrenException, AccessDeniedException, 
        LinkedObjectNotFoundException, ObjectLockedException;
    
    
}

