/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/WebdavSession.java,v 1.19 2001/08/25 14:29:00 dirkv Exp $
 * $Revision: 1.19 $
 * $Date: 2001/08/25 14:29:00 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.webdav.lib;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.net.MalformedURLException;
import org.apache.util.HttpURL;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.ConnectionInterceptor;
import org.apache.commons.httpclient.StreamInterceptor;

/**
 * This WebdavSession class is for the session management of WebDAV clients. 
 * This class saves and restores the requested client.
 * 
 * Although this class is thread safe, it should only be accessed by one
 * concurrent thread, since the underlying protocol, HTTP, is not multiplexed.
 * If simultaneous operations are needed, it is recommended to create 
 * additional threads, each having its own associated WebDAV client.
 *
 * Clients that use persistent connections SHOULD limit the number of
 * simultaneous connections that they maintain to a given server. A
 * single-user client SHOULD NOT maintain more than 2 connections with
 * any server or proxy. A proxy SHOULD use up to 2*N connections to
 * another server or proxy, where N is the number of simultaneously
 * active users. These guidelines are intended to improve HTTP response
 * times and avoid congestion.
 *
 * @author <a href="mailto:jericho@thinkfree.com">Park, Sung-Gu</a>
 */
public abstract class WebdavSession
    implements ConnectionInterceptor {


    // -------------------------------------------------------  Constructors


    /**
     * Default constructor.
     */
    public WebdavSession() {
        super();
    }


    // ---------------------------------------------------- Instance Variables


    /**
     * The Http client instance.
     */
    protected HttpClient client;


    /**
     * Debug level.
     */
    protected int debug = 0;


    // ------------------------------------------------------------- Properties


    /**
     * Set debug level.
     */
    public void setDebug(int debug) {
        this.debug = debug;
        if (client != null) {
            client.setDebug(debug);
        }
    }


    // ------------------------------------------------------  Public methods


    /**
     * Get a <code>HttpClient</code> instance.
     * This method returns a new client instance for the first time.
     * And it is saved util it's closed or reset.
     *
     * @param httpURL The http URL to connect.  only used the authority part.
     * @return An instance of <code>HttpClient</code>.
     * @exception IOException
     */
    public HttpClient getSessionInstance(HttpURL httpURL)
        throws IOException {

        return getSessionInstance(httpURL, false);
    }


    /**
     * Get a <code>HttpClient</code> instance.
     * This method returns a new client instance, when reset is true.
     *
     * @param httpURL The http URL to connect.  only used the authority part.
     * @param reset The reset flag to represent whether the saved information
     *              is used or not.
     * @return An instance of <code>HttpClient</code>.
     * @exception IOException
     */
    public HttpClient getSessionInstance(HttpURL httpURL, boolean reset)
        throws IOException {
        
        String authority = httpURL.getAuthority();
        if (reset)
            client = null;
        if (client == null) {
            client = new HttpClient();
            client.setDebug(debug);        
            // Set a state which allows lock tracking
            client.setState(new WebdavState());
            client.startSession(httpURL.getHost(), httpURL.getPort());
            String userName = httpURL.getUserName();
            if (userName != null && userName.length() > 0) {
                String password = httpURL.getPassword();
                client.setCredentials(new Credentials(userName, password));
            }
            client.setConnectionInterceptor(this);
        }

        return client;
    }

   
    /**
     * Close an session and delete the connection information.
     *
     * @param client The HttpClient instance.
     * @exception IOException Error in closing socket.
     */
    public void closeSession()
        throws IOException {
        if (this.client!=null) {
            this.client.endSession();
            // Remove the progress listener.
            // webdavClient.getProgressUtil().addProgressListener(this);
            this.client = null;
        }
    }


    /**
     * Close an session and delete the connection information.
     *
     * @param client The HttpClient instance.
     * @exception IOException Error in closing socket.
     * @deprecated Replaced by closeSession()
     */
    public synchronized void closeSession(HttpClient client)
        throws IOException {
        closeSession();
    }


    /**
     * Connect.
     */
    public void connect() {
        if (debug == 9) {
            System.err.println("CLIENT: connected.");
        }
    }


    /**
     * Disconnect.
     */
    public void disconnect() {
        if (debug == 9) {
            System.err.println("CLIENT: disconnected.");
        }
    }


    /**
     * Retry.
     * 
     * @return boolean true if a retry should be attempted
     */
    public boolean retry(int status) {
        if (debug == 9) {
            System.err.println("CLIENT: retrying.");
        }
        return false;
    }


    /**
     * Recieved an informational status code.
     * 
     * @return boolean true if a retry should be attempted
     */
    public boolean info(int status, Hashtable headers) {
        if (debug == 9) {
            System.err.println("CLIENT: Status: " + status + "\nHeaders: ");
            Enumeration keys = headers.keys();
            while (keys.hasMoreElements()) {
                String header = (String) keys.nextElement();
                String value = (String) headers.get(header);
                System.err.println("\t" + header + ": " + value);
            }
        }
        return false;
    }


    /**
     * Unexpected error.
     * 
     * @param status Status code; can be equal to -1 if status code is not 
     * known
     * @param e Underlying exception; can be null
     * @return boolean true if processing of the request should be stopped
     */
    public boolean error(int status, Exception e) {
        if (debug == 9) {
            System.err.println("CLIENT: Status: " + status + ", Exception: " +
                               e.getMessage());
        }
        return false;
    }


    /**
     * Sent request.
     */
    public void sentRequest() {
        if (debug == 9) {
            System.err.println("CLIENT: sent request.");
        }
    }


    /**
     * Received response.
     */
    public void receivedResponse() {
        if (debug == 9) {
            System.err.println("CLIENT: received response.");
        }
    }


    /**
     * Received expectation.
     */
    public void receivedExpectation() {
        if (debug == 9) {
            System.err.println("CLIENT: received expectation.");
        }
    }

    /**
     * Authenticate.
     */
    public void requiredAuthentication() {
        if (debug == 9) {
            System.err.println("CLIENT: required authentication.");
        }
    }


    /**
     * Authenticate.
     */
    public void authenticate() {
        if (debug == 9) {
            System.err.println("CLIENT: authenticating.");
        }
    }


    /**
     * Progressing by the progress event.
     *
     * @param pe The progress event.
     */
    /*
    public void progressing(ProgressEvent pe) {
        if (debug > 3)
            System.err.println("[EVENT/WebdavSession] " +
                               "action:" + pe.getAction() +
                               ((pe.getResourceName() == null) ? "" :
                               ", resource:" + pe.getResourceName()) +
                               ", soMany:" + pe.getSoMany() +
                               ", remained:" + pe.getRemainedSize() +
                               ", total:" + pe.getTotalSize());
        getProgressUtil().fireProgress(pe);
    }
    */


    /**
     * Get the utility of this progress event and listener.
     *
     * @return ProgressUtil
     */
    /*
    public ProgressUtil getProgressUtil() {
        return sessionProgress;
    }
    */
}
